import networkx as nx
from aiogram import Bot, F, Router
from aiogram.fsm.context import FSMContext
from aiogram.fsm.state import State, StatesGroup
from aiogram.types import (
    CallbackQuery,
    InlineKeyboardButton,
    InlineKeyboardMarkup,
    KeyboardButton,
    Message,
    ReplyKeyboardMarkup,
    ReplyKeyboardRemove,
)
from aiogram.utils.formatting import Bold, Text, as_list, as_marked_section

# Создаем граф
G = nx.Graph()

# Создаем точки графа, точка соответствует игровой локации
G.add_node('start')
G.add_node('A')
G.add_node('B')
G.add_node('C')
G.add_node('D')
G.add_node('end')

# Связываем точки ребрами создавая связи между локациями
G.add_edge('start', 'A')
G.add_edge('start', 'B')
G.add_edge('A', 'C')
G.add_edge('A', 'D')
G.add_edge('D', 'end')

router = Router()


class GameState(StatesGroup):
    quest = State()


@router.message(F.text.lower() == 'exit game')
async def exit_games(message: Message, state: FSMContext):
    """Обработчик завершения игры"""
    await state.clear()
    await message.reply('Game is finished', reply_markup=ReplyKeyboardRemove())


async def get_content(place: str, backward: str, forward: list[str]) -> Text:
    """Возвращает форматированный текс ответа бота"""
    if forward != ['None']:
        f_text = 'Можете продолжить путь'
    else:
        f_text = 'Вы пришли в тупик, поищите другой путь'

    content = as_list(
        as_marked_section(
            Bold('Вы находитесь в локации:'),
            place,
            marker='*️⃣  ',
        ),
        as_marked_section(
            Bold(
                'Вы можете вернуться назад в локацию'
                if backward
                else 'Вы в начале, пути назад нет'
            ),
            backward,
            marker='⏪ ',
        ),
        as_marked_section(
            Bold(f_text),
            *forward,
            marker='⏩ ',
        ),
        sep='\n\n',
    )
    print(type(content))
    return content


async def get_keyboard(backward: list[str], forward: str) -> InlineKeyboardMarkup:
    """Возвращает инлайн клавиатуру в зависимости от локации"""
    buttons = []

    if backward is not None:
        buttons.append(
            [
                InlineKeyboardButton(
                    text=f'◀️◀️◀️ Вернуться в {backward}',
                    callback_data=f'bacward_{backward}',
                )
            ],
        )
    if forward is not None:
        for f in forward:
            buttons.append(
                [
                    InlineKeyboardButton(
                        text=f'Идти в {f} ▶️▶️▶️', callback_data=f'forward_{f}'
                    )
                ],
            )

    keyboard = InlineKeyboardMarkup(inline_keyboard=buttons)
    return keyboard


@router.message(F.text.lower() == 'quest')
async def start_quest(message: Message, state: FSMContext):
    """Обработчик старта игры"""
    quest_kb = [
        [KeyboardButton(text='Exit Game')],
    ]

    await state.set_state(GameState.quest)

    data = {'backward': None, 'place': 'start', 'forward': ['A', 'B']}

    await state.set_data(data)

    await message.reply('Start quest', reply_markup=ReplyKeyboardRemove())
    await message.reply(
        'Make your move',
        reply_markup=ReplyKeyboardMarkup(keyboard=quest_kb, resize_keyboard=True),
    )
    await make_move(message, state)


@router.message(GameState.quest, F.text)
async def make_move(message: Message, state: FSMContext):
    """Обработчик смены локации в игре"""
    data = await state.get_data()

    keyboard = await get_keyboard(data['backward'], data['forward'])
    content = await get_content(data['place'], data['backward'], data['forward'])

    await message.answer(**content.as_kwargs(), reply_markup=keyboard)


@router.callback_query(GameState.quest)
async def check_answer(callback: CallbackQuery, state: FSMContext, bot: Bot):
    """Обработчик колбэков на локациях"""
    data = await state.get_data()
    data['backward'] = data['place']
    data['place'] = (callback.data.split('_'))[1]
    data['forward'] = list(G.neighbors(data['place']))
    data['forward'].remove(data['backward'])

    await state.update_data(data)
    if data['place'] == 'end':
        await callback.answer(text='Вы дошли до конца!!!', show_alert=True)
        await start_quest(callback.message, state)
        return

    content = await get_content(
        data['place'], data['backward'], data['forward'] or ['None']
    )
    keyboard = await get_keyboard(data['backward'], data['forward'])

    await bot.send_message(
        **content.as_kwargs(), chat_id=callback.from_user.id, reply_markup=keyboard
    )
    await callback.answer()
